import sys
import collections
from optparse import OptionParser,OptionGroup
import copy
from modules.LDUtil import LDIO,Util
import numpy
from modules.CMH import CMHReader

#Author: Martin Kapun & Robert Kofler
#########################################################   HELP   #########################################################################
usage="python %prog --candidates candidates.cmh -snps full.cmh --bins 10 --maxdist  1000 --measure median --output candidates.ld"
parser = OptionParser(usage=usage)
group=OptionGroup(parser,"""				
H E L P :
_________

Description:
This script uses sync files with any statistic (P-value, allele frequency change, FST,etc.) in the last column as input. Based on the parameters --bins and --maxdist, the script averages the statistic of SNPs around a candidate. --bins describes the total numbers of bins generated and --maxdist describes the maximum distance in one direction until which adjacent SNPs are considered. Thus, the bin-size calculates as follows: bins/2*maxdist. Note that the script by default log-scales the statistic, which only applies to P-Values. Thus if you use any other statistic disable log-scaling by setting --log False. Additionally you can choose between calcultating the geometric mean and the median with the parameter --measure. Note that the candidate is excluded from the data.

Input: 
--candidates and --snps:
Any sync file with a statistic in the last column. 

Output:
A tab delimited with two columns for the chromosome and position followed by columns for every bin starting from negative to positive 
E.g. if you set --bin 4 and --maxdist 100, the columns would correspond to the following bins and relative positions to the candidate:
Col1: Chromosome 
Col2: Position 
Col3: averaged statistic of all SNPs in a distance of -100 to -50 basepairs away from the candidate
Col4: averaged statistic of all SNPs in a distance of -50 to -0 basepairs away from the candidate
Col5: averaged statistic of all SNPs in a distance of 0 to 50 basepairs away from the candidate
Col6: averaged statistic of all SNPs in a distance of 50 to 100 basepairs away from the candidate

""") 
#########################################################   CODE   #########################################################################

parser.add_option("-c", "--candidates", dest="cand", help="*.sync or cmh output file")
parser.add_option("-s", "--snps", dest="snps", help="cmh output with all SNPs")
parser.add_option("-b", "--bins", dest="bins", help="number of bins")
parser.add_option("-m", "--maxdist", dest="maxdist", help="maximum distance to candidate.")
parser.add_option("-o", "--output", dest="out", help="outputfile for boxplot")
parser.add_option("--measure", dest="measure", help="What should be calculated, median (median), geometric mean (gm)")
parser.add_option("-l", "--log", dest="log", help="geometric mean or Arithmetic mean, Boolean!!", default=True)
parser.add_option_group(group)
(options, args) = parser.parse_args()

# load candidates
chrh,candl = LDIO.read_candidatehash(options.cand, int(options.maxdist) )

# iterate of SNPs
for snp in CMHReader(options.snps):
	chr=snp.chr
	pos=snp.pos
	candidates=chrh[chr][pos]
	for cand in candidates:
		cand.appendSNP(snp)

# iterate of candidates
ofh=open(options.out,"w")
for cand in candl:
	bins=cand.distributeToBins(int(options.bins))
	toprint=[]
	toprint.append(cand.chr)
	toprint.append(cand.pos)
	toprint.append(cand.pvalue)
	for t in bins:
		value=0
		pvalar=[snp.pvalue for snp in t]
		if options.measure == "median":
			if options.log==True:
				value=Util.median(pvalar,"True")
			else:
				value=Util.median(pvalar,"False")
		elif options.measure=="gm":
			value=Util.geometricmean(pvalar)
		else:
			raise Exception("Unknown option for --measure: "+options.measure)
	
		if value is None:
			value="na"
		toprint.append(value)
	
	s="\t".join(map(str,toprint))
	ofh.write(s+"\n")

